//*****************************************************************************
// Projeto  : Teste GPS externo com placa PE-STM32 by Arne
// Arquivo  : main.c
// Autor    : Eng. Márcio José Soares
// Versão   : V1.0
// Data     : 11/03/2023
// Descrição: Estudos com STM32F103C8T6 através da placa de estudos PE-STM32
//
// IDE      : SW4STM32 (Eclipse - Linux)
// Gravador : STLink V2
// Gravação : ST-FLASH (Linux)
//
//*****************************************************************************
// Disposição dos pinos:
//
//	Port A:
//		PA0	 - D4   - dado bit4 LCD
//		PA1  - D5   - dado bit5 LCD
//		PA2  - D6   - dado bit6 LCD
//		PA3  - D7   - dado bit7 LCD
//
//	Port B:
//		PB10 - TX3  - GPS RX 	- USART3
//      PB11 - RX3  - GPS TX 	- USART3
//		PB13 - RS   - LCD
//		PB14 - RW   - LCD
//		PB15 - EN   - LCD
//
//  Port C:
//		PC13 - LED  - Live
//
//*****************************************************************************
// Últimas alterações:
//	em 11/03/2023:
//		- criado esse projeto
//
//*****************************************************************************

//*****************************************************************************
// Arquivos incluídos
//*****************************************************************************
#include "main.h"
//*****************************************************************************
// Funções do módulo
//*****************************************************************************

//*******************************************************************************
// Init pins - configura pinos para uso
//
// Entradas - nenhuma
// Saídas   - nenhuma
//*******************************************************************************
void init_pins(void){

	GPIO_InitTypeDef GPIO_InitStructMain;

	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE);
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC, ENABLE);

	GPIO_InitStructMain.GPIO_Pin = LED_Live;
	GPIO_InitStructMain.GPIO_Mode = GPIO_Mode_Out_PP;//GPIO_Mode_Out_OD;
	GPIO_InitStructMain.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_Init(PORTLED, &GPIO_InitStructMain);

	GPIO_ResetBits(PORTLED, LED_Live);

}

//*******************************************************************************
// Init vars - inicia variáveis
//
// Entradas - nenhuma
// Saídas   - nenhuma
//*******************************************************************************
void init_vars(void){

	LED_live = 0;
	autoriza = 0;
	time500ms = 0;

	counter  = 0;

}

//*******************************************************************************
// Função main - função principal
//
// Entradas - nenhuma
// Saídas   - nenhuma (zero apenas)
//*******************************************************************************
int main(void){

	init_pins();								//inicia pinos extras LEDs e botão de entrada
	init_vars();								//inicia variáveis globais
	DelayInit();								//inicia delay

	usart3_init();								//inicia usart3 - 4800, 8, n, 1 - GPS
	DelayMs(1000);								//aguarda LCD iniciar

	//configura LCD
	lcd16x2_init(LCD16X2_DISPLAY_ON_CURSOR_ON_BLINK_OFF);	//inicia LCD

	//envia msg via lcd
	lcd16x2_clrscr();
	lcd16x2_gotoxy(2,0);							//posiciona na linha 1, coluna 2
	lcd16x2_puts("BEM-VINDO AO");					//escreve a partir da última posição
	lcd16x2_gotoxy(2,1);							//posiciona na linha 2, coluna 1
	lcd16x2_puts("LAB do Arne");					//escreve a partir da última posição

	DelayMs(1000);									//aguarda
	timerTIM2_init(1);								//ativa timer 1 com int

	lcd16x2_clrscr();
	lcd16x2_gotoxy(2,0);							//posiciona na linha 1, coluna 2
	lcd16x2_puts("Aguardando");						//escreve a partir da última posição
	lcd16x2_gotoxy(1,1);							//posiciona na linha 2, coluna 1
	lcd16x2_puts("GPS iniciar...");					//escreve a partir da última posição

	while(!autoriza);								//aguarda GPS iniciar
	autoriza = 0;									//impede entrada antes de escrevr no LCD

	lcd16x2_clrscr();
	lcd16x2_gotoxy(4,0);							//posiciona na linha 1, coluna 4
	lcd16x2_puts("HORA GPS");						//escreve a partir da última posição

	while(TRUE){
		if(autoriza){								//tem hora certa??
			autoriza = 0;
			lcd16x2_gotoxy(4,1);					//posiciona na linha 2, coluna 4
			lcd16x2_puts(myHour);					//envia string com hora
		}
	}
}
