/*
 * usart_STM.c
 *
 *  Created on: 24 de set de 2018
 *      Author: arne_
 *
 *  Contém funções de controle da USART
 *  linha STM32F103
 *
 *  Desenvolvido por Eng. Márcio José Soares
 */

//******************************************************************************
// Arquivos incluídos no módulo
//******************************************************************************
#include "../inc/usart_STM.h"

//*******************************************************************************
// Funções do módulo
//*******************************************************************************
#ifdef __USE_USART1__
//*******************************************************************************
// USART 1
//*******************************************************************************
//*******************************************************************************
// usart1_init - configura pinos PB6 e PB7 - usart 1 - para uso
//
// Entradas - nenhuma
// Saídas   - nenhuma
//*******************************************************************************
void usart1_init(void)
{
	USART_InitTypeDef usartConfig;
	GPIO_InitTypeDef gpioConfig;
	//NVIC_InitTypeDef NVIC_InitStructure;

	RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1 | RCC_APB2Periph_GPIOA | RCC_APB2Periph_AFIO, ENABLE);

	//PB6 = USART1.TX => Alternative Function Output
	gpioConfig.GPIO_Pin = GPIO_Pin_9;
	gpioConfig.GPIO_Speed = GPIO_Speed_10MHz;
	gpioConfig.GPIO_Mode = GPIO_Mode_AF_PP;
	GPIO_Init(GPIOA, &gpioConfig);
	//PB7 = USART1.RX => Input
	gpioConfig.GPIO_Pin = GPIO_Pin_10;
	gpioConfig.GPIO_Speed = GPIO_Speed_10MHz;
	gpioConfig.GPIO_Mode = GPIO_Mode_IN_FLOATING;
	GPIO_Init(GPIOA, &gpioConfig);

	USART_Cmd(USART1, ENABLE);

	usartConfig.USART_BaudRate = 9600;
	usartConfig.USART_WordLength = USART_WordLength_8b;
	usartConfig.USART_StopBits = USART_StopBits_1;
	usartConfig.USART_Parity = USART_Parity_No;
	usartConfig.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;
	usartConfig.USART_HardwareFlowControl = USART_HardwareFlowControl_None;

	USART_Init(USART1, &usartConfig);					//configura usart 1
	USART_ITConfig(USART1, USART_IT_RXNE, ENABLE);		//habilita interrup��o RXNE
	NVIC_EnableIRQ(USART1_IRQn);						//Habilita interrup��o global para USART1

}

//*******************************************************************************
// usart1_putchar - envia 1 byte via usart
//
// Entradas - dado a enviar
// Saídas   - nenhuma
//*******************************************************************************
void usart1_putchar(char data)
{

	USART_SendData(USART1, data);		//envia dado
	while(USART_GetFlagStatus(USART1, USART_FLAG_TC) == RESET);	//aguarda liberar
}

//*******************************************************************************
// usart1_send2 - envia dados via usart - dado é variável no programa
//
// Entradas - buffer a enviar
// Saídas   - caracter recebido
//*******************************************************************************
void usart1_buf_sendR(char pucBuffer[])
{
    uint16_t i;
    //unsigned long ulCount = (unsigned long)strlen(pucBuffer);

    if(strlen(pucBuffer) > 0)
    {
    	for(i=0; i< strlen(pucBuffer); i++)
    	{
    		//USART_SendData(USART1, pucBuffer[i]);
    		//while(USART_GetFlagStatus(USART1, USART_FLAG_TC) == RESET);
    		usart1_putchar(pucBuffer[i]);
    	}
    }
}

//*******************************************************************************
// usart1_send - envia dados via usart - dado é constante no programa
//
// Entradas - buffer a enviar e tamanho do buffer
// Saídas   - caracter recebido
//*******************************************************************************
void usart1_buf_send(const unsigned char pucBuffer[], unsigned long ulCount)
{
    unsigned long i;

    if(ulCount > 0)
    {
    	for(i=0; i< ulCount; i++)
    	{
    		//USART_SendData(USART1, pucBuffer[i]);
    		//while(USART_GetFlagStatus(USART1, USART_FLAG_TC) == RESET);
    		usart1_putchar((char)pucBuffer[i]);
    	}
    }
}

//*******************************************************************************
// usart1_ReadByteSync - lê dados da usart1
//
// Entradas - tipo definido pelo usuário - usart desejada
// Saídas   - caracter recebido
//*******************************************************************************
unsigned char usart1_ReadByteSync(USART_TypeDef *USARTx)
{
    while ((USARTx->SR & USART_SR_RXNE) == 0)
    {
    }

    return (unsigned char)USART_ReceiveData(USARTx);
}

//*******************************************************************************
// usart1_IRQHandle - lê dados da usart1
//
// Entradas - nenhuma
// Saídas   - nenhuma
//*******************************************************************************
void USART1_IRQHandler(void)
{
	char mychar;
    /* RXNE handler */
    if(USART_GetITStatus(USART1, USART_IT_RXNE) != RESET)	//verifica se tem dado
	//if ((USART1->SR & USART_FLAG_RXNE) != (u16)RESET)
    {
    	mychar = (char)USART_ReceiveData(USART1);			//retira char
    	//dadoHEX = (uint8_t)mychar;
        USART_SendData(USART1, mychar);						//envia eco
    }

    /* ------------------------------------------------------------------------------- */
    /* outros controles para usart1 podem ser inseridos a partir daqui ...             */
}
#endif

#ifdef __USE_USART2__
//*******************************************************************************
// USART 2
//*******************************************************************************
//*******************************************************************************
// usart2_init - configura pinos PA2 e PA3 - usart 2 - para uso
//
// Entradas - nenhuma
// Saídas   - nenhuma
//*******************************************************************************
void usart2_init(void)
{
	USART_InitTypeDef usart2Config;
	GPIO_InitTypeDef gpio2Config;
	//NVIC_InitTypeDef NVIC_InitStructure;

	RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2 | RCC_APB2Periph_GPIOA | RCC_APB2Periph_AFIO, ENABLE);

	//PA2 = USART2.TX => Alternative Function Output
	gpio2Config.GPIO_Pin = GPIO_Pin_2;
	gpio2Config.GPIO_Speed = GPIO_Speed_10MHz;
	gpio2Config.GPIO_Mode = GPIO_Mode_AF_PP;
	GPIO_Init(GPIOA, &gpio2Config);
	//PA3 = USART2.RX => Input
	gpio2Config.GPIO_Pin = GPIO_Pin_3;
	gpio2Config.GPIO_Speed = GPIO_Speed_10MHz;
	gpio2Config.GPIO_Mode = GPIO_Mode_IN_FLOATING;
	GPIO_Init(GPIOA, &gpio2Config);

	USART_Cmd(USART2, ENABLE);

	//PCLK1 dividido por 8... por 4 também funciona!
	// 72 / 8 = 9MHz
	usart2Config.USART_BaudRate = 300;
	usart2Config.USART_WordLength = USART_WordLength_8b;
	usart2Config.USART_StopBits = USART_StopBits_1;
	usart2Config.USART_Parity = USART_Parity_No;
	usart2Config.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;
	usart2Config.USART_HardwareFlowControl = USART_HardwareFlowControl_None;

	USART_Init(USART2, &usart2Config);					//configura usart 1
	USART_ITConfig(USART2, USART_IT_RXNE, ENABLE);		//habilita interrupção RXNE
	NVIC_EnableIRQ(USART2_IRQn);						//Habilita interrupção global para USART1

}

//*******************************************************************************
// usart2_putchar - envia 1 byte via usart
//
// Entradas - dado a enviar
// Saídas   - nenhuma
//*******************************************************************************
void usart2_putchar(char data)
{

	USART_SendData(USART2, data);		//envia dado
	while(USART_GetFlagStatus(USART2, USART_FLAG_TC) == RESET);	//aguarda liberar
}

//*******************************************************************************
// usart2_send2 - envia dados via usart -
//
// Entradas - buffer a enviar
// Saídas   - caracter recebido
//*******************************************************************************
void usart2_buf_sendR(char pucBuffer[])
{
    uint16_t i;
    //unsigned long ulCount = (unsigned long)strlen(pucBuffer);

    if(strlen(pucBuffer) > 0)
    {
    	for(i=0; i< strlen(pucBuffer); i++)
    	{
    		//USART_SendData(USART2, pucBuffer[i]);
    		//while(USART_GetFlagStatus(USART2, USART_FLAG_TC) == RESET);
    		usart2_putchar(pucBuffer[i]);
    	}
    }
}

//*******************************************************************************
// usart2_send - envia dados via usart - dado é constante no programa
//
// Entradas - buffer a enviar e tamanho do buffer
// Saídas   - caracter recebido
//*******************************************************************************
void usart2_buf_send(const unsigned char pucBuffer[], unsigned long ulCount)
{
    unsigned long i;

    if(ulCount > 0)
    {
    	for(i=0; i< ulCount; i++)
    	{
    		//USART_SendData(USART1, pucBuffer[i]);
    		//while(USART_GetFlagStatus(USART1, USART_FLAG_TC) == RESET);
    		usart2_putchar((char)pucBuffer[i]);
    	}
    }
}

//*******************************************************************************
// usart2_ReadByteSync - lê dados da usart1
//
// Entradas - tipo definido pelo usuário - usart desejada
// Saídas   - caracter recebido
//*******************************************************************************
unsigned char usart2_ReadByteSync(USART_TypeDef *USARTx)
{
    while ((USARTx->SR & USART_SR_RXNE) == 0)
    {
    }

    return (unsigned char)USART_ReceiveData(USARTx);
}

//*******************************************************************************
// usart2_IRQHandle - lê dados da usart2
//
// Entradas - nenhuma
// Saídas   - nenhuma
//*******************************************************************************
void USART2_IRQHandler(void)
{
	char mychar = 0;
    /* RXNE handler */
    if(USART_GetITStatus(USART2, USART_IT_RXNE) != RESET){		//verifica se tem dado
    	mychar = (char)USART_ReceiveData(USART2);				//retira char
       	if(mychar == 0x12){										//se está conectado, mantem
    		termPres = 0x01;
    		termPresCount = 0;
       	}

   		recchar = mychar;

   		if(mychar == TECLA_XE){
   			if(!getRevProg && !getHorario && !getPlanos && !getRevPlan){	//teclou XE e não é revisão ou acerto de relogio
   				getBasicProg = 1;				//entra na programação
#ifdef __DEBUG_RS232__
				temp_debug_ja_mostrou = 0;
#endif
   				//recchar = 1;
   			}
   			else{								//teclo XE e é revisão
   				if(!getHorario){
#ifdef __DEBUG_RS232__
#ifdef __DEBUG_PROG_BASE__
   					strcpy(TxBuffer, "\r\nMandou salvar prog. basica");
   					usart3_buf_sendR(TxBuffer);
#endif
#endif
   					if(getRevProg)				//se revisão prog. base
   						savePB = 1;
   					else if(getRevPlan)			//se revisão plan
   						savePL = 1;

   				}
   			}
   		}

   		if(mychar == TECLA_AC){
   			if(getRevProg && !autRevisa){		//é revisão da prog. base e pediu para alterar
   				alteraProgBas();
   			}
   			else if(getRevPlan && !autRevisa){	//é revisão do plano e pediu para alterar
   				alteraPlan();
   			}
       	}

   		if(mychar == TECLA_AL){
   			if(getRevProg){						//pediu para revisar prog base
   			   	revisaProgBas();
   			}

   			if(getRevPlan){						//pediu para revisar plano
   				revisaPlan();
   			}
   		}

       	if(mychar == TECLA_E){
       		if(getRevProg && !autRevisa){		//teclou enter dentro da revisão prog. base
       			avancaProgBas();
       		}

       		if(getRevPlan && !autRevisa){		//teclou enter dentro da revisão planos
       			avancaPlan();
       		}
       	}

       	if(mychar == TECLA_MF){					//por enquanto usado para acertar relógio
       		getHorario = 1;
       	}

       	if(mychar == TECLA_XMF){				//por enquanto entra na prog dos planos
       		if(!getBasicProg && !getRevProg && !getHorario && !autRevisa){ //nada acontecendo
       			getPlanos = 1;
#ifdef __DEBUG_RS232__
				temp_debug_ja_mostrou = 0;
#endif
       		}
       	}

       	if(getBasicProg || getHorario || getPlanos || (getRevProg && autRevisa)){
       		if(mychar >= 0x30 && mychar <= 0x39){					//eco apenas das teclas 0 a 9
       			usart2_putchar(mychar);
       		}
       	}

#ifdef __DEBUG_RS232__
       	if(mychar != 0x12){
       		bin_hex(mychar, &mhigh, &mlow);
       		usart3_putchar(mhigh);
       		usart3_putchar(mlow);
       		usart3_putchar(' ');
       	}
#endif

    }

    /* ------------------------------------------------------------------------------- */
    /* outros controles para usart2 podem ser inseridos a partir daqui ...             */
}
#endif


#ifdef __USE_USART3__
//*******************************************************************************
// USART 3
//*******************************************************************************
//*******************************************************************************
// usart3_init - configura pinos PB11 e PB10 - usart 3 - para uso
//
// Entradas - nenhuma
// Saídas   - nenhuma
//*******************************************************************************
void usart3_init(void)
{
	USART_InitTypeDef usartConfig;
	GPIO_InitTypeDef gpioConfig;
	//NVIC_InitTypeDef NVIC_InitStructure;

	RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART3 | RCC_APB2Periph_GPIOB | RCC_APB2Periph_AFIO, ENABLE);

	//PB10 = USART3.TX => Alternative Function Output
	gpioConfig.GPIO_Pin = GPIO_Pin_10;
	gpioConfig.GPIO_Speed = GPIO_Speed_10MHz;
	gpioConfig.GPIO_Mode = GPIO_Mode_AF_PP;
	GPIO_Init(GPIOB, &gpioConfig);
	//PB11 = USART1.RX => Input
	gpioConfig.GPIO_Pin = GPIO_Pin_11;
	gpioConfig.GPIO_Speed = GPIO_Speed_10MHz;
	gpioConfig.GPIO_Mode = GPIO_Mode_IN_FLOATING;
	GPIO_Init(GPIOB, &gpioConfig);

	USART_Cmd(USART3, ENABLE);

	usartConfig.USART_BaudRate = 4800;
	usartConfig.USART_WordLength = USART_WordLength_8b;
	usartConfig.USART_StopBits = USART_StopBits_1;
	usartConfig.USART_Parity = USART_Parity_No;
	usartConfig.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;
	usartConfig.USART_HardwareFlowControl = USART_HardwareFlowControl_None;

	USART_Init(USART3, &usartConfig);					//configura usart 3
	USART_ITConfig(USART3, USART_IT_RXNE, ENABLE);		//habilita interrupção RXNE
	NVIC_EnableIRQ(USART3_IRQn);						//Habilita interrupção global para USART3

}

//*******************************************************************************
// usart3_putchar - envia 1 byte via usart3
//
// Entradas - dado a enviar
// Saídas   - nenhuma
//*******************************************************************************
void usart3_putchar(char data)
{

	USART_SendData(USART3, data);								//envia dado
	while(USART_GetFlagStatus(USART3, USART_FLAG_TC) == RESET);	//aguarda liberar
}

//*******************************************************************************
// usart3_buf_sendR - envia dados via usart - dado � vari�vel no programa
//
// Entradas - buffer a enviar
// Saídas   - caracter recebido
//*******************************************************************************
void usart3_buf_sendR(char pucBuffer[])
{
    uint16_t i;
    //unsigned long ulCount = (unsigned long)strlen(pucBuffer);

    if(strlen(pucBuffer) > 0)
    {
    	for(i=0; i< strlen(pucBuffer); i++)
    	{
    		//USART_SendData(USART1, pucBuffer[i]);
    		//while(USART_GetFlagStatus(USART1, USART_FLAG_TC) == RESET);
    		usart3_putchar(pucBuffer[i]);
    		//DelayMs(5);
    	}
    }
}

//*******************************************************************************
// usart3_send - envia dados via usart3 - dado � constante no programa
//
// Entradas - buffer a enviar e tamanho do buffer
// Saídas   - caracter recebido
//*******************************************************************************
void usart3_buf_send(const unsigned char pucBuffer[], unsigned long ulCount)
{
    unsigned long i;

    if(ulCount > 0)
    {
    	for(i=0; i< ulCount; i++)
    	{
    		//USART_SendData(USART1, pucBuffer[i]);
    		//while(USART_GetFlagStatus(USART1, USART_FLAG_TC) == RESET);
    		usart3_putchar((char)pucBuffer[i]);
    	}
    }
}

//*******************************************************************************
// usart3_ReadByteSync - l� dados da usart3
//
// Entradas - tipo definido pelo usu�rio - usart desejada
// Saídas   - caracter recebido
//*******************************************************************************
unsigned char usart3_ReadByteSync(USART_TypeDef *USARTx)
{
    while ((USARTx->SR & USART_SR_RXNE) == 0)
    {
    }

    return (unsigned char)USART_ReceiveData(USARTx);
}

//*******************************************************************************
// usart3_IRQHandle - l� dados da usart3
//
// Entradas - nenhuma
// Saídas   - nenhuma
//*******************************************************************************
void USART3_IRQHandler(void)
{
	char mychar;
    /* RXNE handler */
    if(USART_GetITStatus(USART3, USART_IT_RXNE) != RESET)	//verifica se tem dado
    {
    	mychar = (char)USART_ReceiveData(USART3);			//retira char
        GPS_get_data(mychar);								//trata dado recebido do GPS
    }


}
#endif
